<?php

$attachmentHandler = new AttachmentHandler($Config);

class AttachmentHandler {
    private $config;
    private $allowed_ext = ["mp3", "webp", "mp4", "mov", "wmv", "flv", "jpg", "gif", "png", "jpeg", "bmp", "pdf", "doc", "docm", "docx", "dot", "dotx"];
    private $maxsize = 5368709120; // 5GB in bytes
    
    public function __construct($config) {
        $this->config = $config;
    }

    public function uploadAttachment($course_id, $uploadPath) {
        header('Content-Type: application/json; charset=utf-8');
        
        // Set PHP limits for large file uploads
        ini_set('memory_limit', '2G');
        ini_set('max_execution_time', 7200);
        ini_set('post_max_size', '5G');
        ini_set('upload_max_filesize', '5G');

        $Data = ['status' => false, 'msg' => ''];

        try {
            if (!isset($_FILES['file'])) {
                throw new Exception('No file uploaded');
            }

            $file = $_FILES['file'];
            $fileName = $file['name'];
            $fileSize = $file['size'];
            $fileTmp = $file['tmp_name'];
            $fileType = $file['type'];
            $fileError = $file['error'];

            // Validate file size
            if ($fileSize > $this->maxsize) {
                throw new Exception('File size exceeds limit (5GB)');
            }

            // Get file extension and validate
            $fileExt = strtolower(pathinfo($fileName, PATHINFO_EXTENSION));
            if (!in_array($fileExt, $this->allowed_ext)) {
                throw new Exception('File type not allowed');
            }

            // Create module folder if it doesn't exist
            $moduleFolder = $uploadPath . '/' . $course_id;
            if (!is_dir($moduleFolder)) {
                if (!mkdir($moduleFolder, 0755, true)) {
                    throw new Exception('Failed to create module directory');
                }
            }

            // Generate unique filename
            $newFileName = uniqid() . '_' . $fileName;
            $destination = $moduleFolder . '/' . $newFileName;

            // Move uploaded file
            if ($fileSize > 5242880) { // 5MB
                // Use chunk copy for large files
                if (!$this->copyLargeFile($fileTmp, $destination)) {
                    throw new Exception('Failed to copy large file');
                }
            } else {
                if (!move_uploaded_file($fileTmp, $destination)) {
                    throw new Exception('Failed to move uploaded file');
                }
            }

            // Prepare response
            $Data['status'] = true;
            $Data['msg'] = 'File uploaded successfully';
            $Data['file_name'] = $newFileName;
            $Data['original_name'] = $fileName;
            $Data['file_size'] = $fileSize;
            $Data['file_type'] = $fileType;

            // Handle video files
            if (in_array($fileExt, ['mp4', 'mov', 'wmv', 'flv'])) {
                $duration = $this->getVideoDuration($destination);
                $Data['duration'] = $duration;
            }

        } catch (Exception $e) {
            $Data['status'] = false;
            $Data['msg'] = $e->getMessage();
            if (isset($file['error'])) {
                $Data['error_code'] = $file['error'];
            }
        }

        return $Data;
    }

    public function deleteAttachment($course_id, $fileName, $uploadPath) {
        $filePath = $uploadPath . '/' . $course_id . '/' . $fileName;
        
        if (file_exists($filePath)) {
            if (unlink($filePath)) {
                return ['status' => true, 'msg' => 'File deleted successfully'];
            }
            return ['status' => false, 'msg' => 'Failed to delete file'];
        }
        
        return ['status' => false, 'msg' => 'File not found'];
    }

    private function copyLargeFile($source, $destination, $chunkSize = 1048576) {
        $fsrc = fopen($source, 'rb');
        $fdest = fopen($destination, 'wb');
        
        while (!feof($fsrc)) {
            if (fwrite($fdest, fread($fsrc, $chunkSize)) === false) {
                return false;
            }
        }
        
        fclose($fsrc);
        fclose($fdest);
        
        return true;
    }

    private function getVideoDuration($videoPath) {
        if (class_exists('FFMpeg\\FFMpeg')) {
            try {
                $ffmpeg = FFMpeg\FFMpeg::create();
                $video = $ffmpeg->open($videoPath);
                $duration = $video->getFormat()->get('duration');
                return $duration;
            } catch (Exception $e) {
                return 0;
            }
        }
        return 0;
    }
}
