<?php

///////////////////////////////////////////////////////////////////////////////
/**
 * Filename: upload-class.php
 * Purpose: Upload FIles
 * Author: TRAIDNT
 * Developer : Ahmed Elsayed
 * Date: 10/11/2009
 */
///////////////////////////////////////////////////////////////////////////////


Class Upload {

    /**
     * array content allow ext list
     * @protected   array
     */
    protected $_allowext;

    /**
     *  to print logo to images if TRUE
     * @protected   bool
     */
    protected $_logoimages;

    /**
     * create thmubs images if true
     * @protected   bool
     */
    protected $_thumbsimages;

    /**
     * thumbs images width
     * @protected   int
     */
    protected $_th_width;

    /**
     * thumbs images hight
     * @protected   int
     */
    protected $_th_hight;

    /**
     * filename prefix
     * @protected   str
     */
    protected $_nameprefix;

    /**
     * max upload file
     * @protected   int
     */
    protected $_maxsize;

    /**
     * list error's
     * @protected   array
     */
    protected $_error;

    /**
     * list disallow mime type
     * @protected   array
     */
    protected $_disallow_mime = array("text/html",
        "text/plain",
        "magnus-internal/shellcgi",
        "application/x-php",
        "text/php",
        "application/x-httpd-php",
        "application/php",
        "magnus-internal/shellcgi",
        "text/x-perl",
        "application/x-perl",
        "application/x-exe",
        "application/exe",
        "application/x-java",
        "application/java-byte-code",
        "application/x-java-class",
        "application/x-java-vm",
        "application/x-java-bean",
        "application/x-jinit-bean",
        "application/x-jinit-applet",
        "magnus-internal/shellcgi",
        "image/svg",
        "image/svg-xml",
        "image/svg+xml",
        "text/xml-svg",
        "image/vnd.adobe.svg+xml",
        "image/svg-xml",
        "text/xml",
    );

    /**
     * file array info include (name,path,type,ext,etc)
     * @protected   array
     */
    protected $_fileinfo;

    protected $_thumbsConfig = array();

    /**
     * site logo path using for print logo
     * @protected   str
     */
    protected $_logopath;

    /**
     * uploading folder
     * @private str
     */
    protected  $_uploadfolder;

    /**
     * Constructor
     * @param   array   list allow ext
     * @param   array   thumbs config
     * @param   int     max size to upload
     * @param   str     uploading folder
     * @param   bool    create thumbs
     * @return  void
     */
    public function __construct($allowext, $thumbsConfig, $maxsize, $uploadfolder, $thumbsimages = true) {
        // تعيين القيم الأساسية
        $this->_allowext = $allowext;
        $this->_thumbsConfig = $thumbsConfig;
        $this->_maxsize = $maxsize;
        $this->_thumbsimages = $thumbsimages;
        
        // معالجة مسار مجلد التحميل
        // print_r($uploadfolder);die();
        $this->_uploadfolder = rtrim($uploadfolder, '/') . '/';
        
        // إنشاء المجلد إذا لم يكن موجوداً
        if (!is_dir($this->_uploadfolder)) {
            if (!mkdir($this->_uploadfolder, 0755, true)) {
                throw new Exception("فشل في إنشاء مجلد التحميل: " . $this->_uploadfolder);
            }
        }
        
        // التحقق من صلاحيات الكتابة
        if (!is_writable($this->_uploadfolder)) {
            throw new Exception("مجلد التحميل غير قابل للكتابة: " . $this->_uploadfolder);
        }
        
        // تهيئة مصفوفة الأخطاء
        $this->_error = array();
    }

    /**
     * Upload File
     * @param   array   file info array #note : this array content data from $_FILES array
     *                  just have this key [type,name,size,tmp]
     * @param   int     max size
     * @param   string  custom file name (optional)
     * @return  full file info array in success OR FALSE in failure
     */
    public function Upload_File($filearray, $maxsize = NULL, $customName = NULL) {
        try {
            // Initialize file info
            $this->_fileinfo = array();
            
            // Get original filename and ensure it's UTF-8
            $originalName = $filearray['name'];
            if (!mb_check_encoding($originalName, 'UTF-8')) {
                $originalName = mb_convert_encoding($originalName, 'UTF-8');
            }
            
            $this->_fileinfo['original_name'] = $originalName;
            $this->_fileinfo['ext'] = strtolower($this->GetExt($originalName));
            $this->_fileinfo['size'] = $filearray['size'];
            $this->_fileinfo['tmp'] = $filearray['tmp'];
            $this->_fileinfo['type'] = $filearray['type'];
            $this->_fileinfo['place'] = $this->_uploadfolder;

            // Generate new filename
            if ($customName) {
                // Use custom name but ensure it's safe
                $baseName = pathinfo($customName, PATHINFO_FILENAME);
                $baseName = $this->sanitizeFileName($baseName);
            } else {
                // Use original name but ensure it's safe
                $baseName = $this->sanitizeFileName(pathinfo($originalName, PATHINFO_FILENAME));
            }
            
            // Add unique identifier to prevent overwriting
            $uniqueId = substr(uniqid(), -8);
            $this->_fileinfo['newname'] = $baseName . '_' . $uniqueId . '.' . $this->_fileinfo['ext'];

            // Create thumbnails directory if needed
            if ($this->is_img($this->_fileinfo['ext']) && $this->_thumbsimages && $this->_thumbsConfig) {
                foreach ($this->_thumbsConfig as $key => $config) {
                    $thumbDir = $this->_uploadfolder . 'thumbs/' . $key . '/';
                    if (!is_dir($thumbDir)) {
                        mkdir($thumbDir, 0755, true);
                    }
                    $this->_fileinfo['thplace'][$key] = $thumbDir . $this->_fileinfo['newname'];
                }
            }

            // Check file extension
            if (!$this->CheckExt($this->_fileinfo['ext'])) {
                throw new Exception('File extension not allowed: ' . $this->_fileinfo['ext']);
            }

            // Validate file size
            if (!$this->CheckSize($this->_fileinfo['size'], $maxsize)) {
                throw new Exception('File size exceeds maximum allowed size');
            }

            // Validate file type security
            if (!$this->validateFileType($this->_fileinfo['tmp'])) {
                throw new Exception('Invalid or potentially dangerous file type');
            }

            // Move uploaded file
            if (!$this->move_uploaded($this->_fileinfo['tmp'], $this->_fileinfo['place'], $this->_fileinfo['newname'])) {
                throw new Exception('Failed to move uploaded file');
            }

            // Generate thumbnails if needed
            if ($this->is_img($this->_fileinfo['ext']) && $this->_thumbsimages && $this->_thumbsConfig) {
                foreach ($this->_thumbsConfig as $key => $config) {
                    if (!$this->createthumb(
                        $this->_fileinfo['place'] . $this->_fileinfo['newname'],
                        $this->_fileinfo['ext'],
                        $this->_fileinfo['thplace'][$key],
                        $config['width'],
                        $config['height']
                    )) {
                        // Log warning but don't fail upload
                        error_log("Failed to create thumbnail: {$key}");
                    }
                }
            }

            return $this->_fileinfo;

        } catch (Exception $e) {
            $this->return_error($e->getMessage());
            return false;
        }
    }

    /**
     * Sanitize filename to prevent directory traversal and invalid characters
     * while preserving Arabic characters
     * @param string $filename Original filename
     * @return string Sanitized filename
     */
    private function sanitizeFileName($filename) {
        // Remove directory traversal attempts
        $filename = basename($filename);
        
        // Convert to UTF-8 if not already
        if (!mb_check_encoding($filename, 'UTF-8')) {
            $filename = mb_convert_encoding($filename, 'UTF-8');
        }
        
        // Replace potentially dangerous characters while preserving Arabic
        $filename = preg_replace('/[^\p{Arabic}\p{L}\p{N}\s\-\_\.]/u', '', $filename);
        
        // Replace multiple spaces with single space
        $filename = preg_replace('/\s+/', ' ', $filename);
        
        // Trim spaces from start and end
        $filename = trim($filename);
        
        // Ensure filename isn't too long (considering UTF-8 characters)
        if (mb_strlen($filename, 'UTF-8') > 255) {
            $ext = pathinfo($filename, PATHINFO_EXTENSION);
            $name = pathinfo($filename, PATHINFO_FILENAME);
            $filename = mb_substr($name, 0, 255 - mb_strlen($ext, 'UTF-8') - 1, 'UTF-8') . '.' . $ext;
        }
        
        // If filename is empty after sanitization, use a timestamp
        if (empty($filename)) {
            $filename = date('Y-m-d_H-i-s');
        }
        
        return $filename;
    }

    /**
     * Validate file type using multiple methods
     * @param string $filepath Path to temporary uploaded file
     * @return bool True if file is safe, false otherwise
     */
    private function validateFileType($filepath) {
        // Check file exists
        if (!file_exists($filepath)) {
            return false;
        }

        // Try using FileInfo extension first
        if (function_exists('finfo_open')) {
            $finfo = finfo_open(FILEINFO_MIME_TYPE);
            $mime_type = finfo_file($finfo, $filepath);
            finfo_close($finfo);
        } 
        // Fallback to mime_content_type if available
        else if (function_exists('mime_content_type')) {
            $mime_type = mime_content_type($filepath);
        }
        // Last resort: use the $_FILES mime type
        else {
            $mime_type = $_FILES['file']['type'];
        }

        // Validate against disallowed MIME types
        if (in_array($mime_type, $this->_disallow_mime)) {
            return false;
        }

        // Additional security checks for specific file types
        if (strpos($mime_type, 'text/') === 0) {
            // Check for PHP code in text files
            $content = file_get_contents($filepath);
            if (preg_match('/<\?php/i', $content)) {
                return false;
            }
        }

        return true;
    }

    /**
     * GET file ext
     * @param   str     file name
     * @return  file ext in success Or False in failure
     */
    public function GetExt($filename) {

        // if file name is not null
        if (!is_null($filename)) {

            // make the file name lower case and crop the ext
            $fileext = strtolower(strrchr(strtolower($filename), '.'));
            // replace the dot (.) with blank
            $fileext = str_replace(".", "", $fileext);
            // return the file ext
            return($fileext);
        } else {
            // return false if file name is null
            return(FALSE);
        }
    }

    /**
     * check if file ext allow in allow ext array or not
     * @param   str     file name
     * @return  BOLL    TRUE if allowed Or FALSE
     */
    public function CheckExt($ext) {

        // check if ext is nul Or allow ext array is not array
        if (!is_null($ext) OR !is_array($this->_allowext)) {
            // check if ext in array of allow ext
            if (in_array($ext, $this->_allowext)) {
                // return true it's mean it is allow ext
                return(TRUE);
            } else {
                // return false it's mean it is disallow ext
                return(FALSE);
            }
        } else {
            // return false if ext param is null or allowext it not array
            return(FALSE);
        }
    }

    /**
     * check if file size large than max upload size or not
     * @param   int     file size
     * @param   int     max file size
     * @return  BOLL    TRUE Or FALSE
     */
    public function CheckSize($filesieze, $maxsize) {

        // if file size is not null and numeric
        if (is_numeric($filesieze) OR !is_null($filesieze) OR !is_null($this->_maxsize)) {
            if (is_null($maxsize)) {
                $max = $this->_maxsize;
            } else {
                $max = $maxsize;
            }

            //check if file size large than the max upload size
            if (round($filesieze) < $max) {
                return(TRUE);
            } else {
                return(FALSE);
            }
        } else {
            // if file size is  null and  not numeric
            return(FALSE);
        }
    }

    /**
     * generate new key
     * @param   void
     * @return  str     key
     */
    public function uniqname() {
        return(substr(md5(uniqid(rand())), 0, 10));
    }

    /**
     * chech file type
     * @param   str     file ext
     * @return  boll    TRUE mean is img
     */
    public function is_img($ext) {
        // list of  images ext
        $picarray = array("jpg", "gif", "png", "jpeg", "bmp", "jpeg");

        // if param ext is not null
        if (!is_null($ext)) {

            // if param ext in images ext array
            if (in_array($ext, $picarray)) {
                // return true to mean it's images
                return(TRUE);
            } else {
                // return false to mean it's not image
                return(FALSE);
            }
        } else {
            // if param ext is  null
            return(FALSE);
        }
    }

    /**
     * generat keywords from filename
     * @param   str     file name
     * @return  array   false mean file name is null
     */
    private function GetFileKeywords($filename) {
        if (!is_null($filename)) {
            $str = trim($filename);
            $str = preg_replace("#[&].{2,7}[;]#sim", " ", $str);
            $str = preg_replace("#[()A°^!\"A§\$%&/{(\[)\]=}?A´`,;.:\-_\#'~+*]#", " ", $str);
            $str = preg_replace("#\s+#sim", " ", $str);
            $arraw = explode(" ", $str);

            foreach ($arraw as $v) {
                $v = trim($v);
                if (strlen($v) < 3 || in_array($v, $this->_allowext))
                    continue;

                $arr[] = $v;
            }

            $final = @implode(",", $arr);

            return($final);
        }
        else {
            return(false);
        }
    }

    /**
     * get full path
     * @param   void
     * @return  str     full path
     */ 
    private function CurrentFullPath() {
        $root     = $_SERVER['DOCUMENT_ROOT']; 
        $self     = $_SERVER['PHP_SELF']; 
        $self_arr = explode('/', $self);
        $self     = '/'.$self_arr[1];
        return $root . $self;//mb_substr($self, 0, -mb_strlen(strrchr($self, "/")));
    }

    /**
     * move uploaded file
     * @param   str     file tmp
     * @param   str     new file place
     * @param   str     new file name
     * @return  boll    TRUE it's success moved
     */
    private function move_uploaded($tmp, $path, $name) {
        try {
            // التأكد من وجود المجلد وإنشائه إذا لم يكن موجوداً
            if (!is_dir($path)) {
                if (!mkdir($path, 0755, true)) {
                    throw new Exception("فشل في إنشاء مجلد التحميل: " . $path);
                }
            }

            // التحقق من صلاحيات الكتابة
            if (!is_writable($path)) {
                throw new Exception("المجلد غير قابل للكتابة: " . $path);
            }

            // نقل الملف
            if (!move_uploaded_file($tmp, $path . $name)) {
                throw new Exception("فشل في نقل الملف المرفوع");
            }

            // التحقق من وجود الملف بعد النقل
            if (!file_exists($path . $name)) {
                throw new Exception("الملف غير موجود بعد النقل: " . $path . $name);
            }

            return true;
        } catch (Exception $e) {
            $this->return_error($e->getMessage());
            return false;
        }
    }

    /**
     * check the weather ot ext dir
     * @param   str     file ext
     * @return  boll    TRUE OR FALSE
     */
    private function ExtDir($ext) { 
        // if param ext is not null
        if (!is_null($ext)) {
            echo $this->_uploadfolder; 
            // if upload folder is not exist create it
            if (!is_dir($this->_uploadfolder)) { 
                $mkdir = @mkdir($this->_uploadfolder, 0777);
                if (!$mkdir) { echo '5';
                    // return false if php can't create upload folder
                    return(FALSE);
                }
            } else { 
                // if ext folder is not exist
                if (!is_dir($this->_uploadfolder)) {

                    // create ext folder
                    $mkdir = @mkdir($this->_uploadfolder, 0777);
                    // create the thumbs folder
                    $mkdir2 = @mkdir($this->_uploadfolder . "/thumbs", 0777);

                    // if php create folder
                    if ($mkdir AND $mkdir2) {
                        return(TRUE);
                    }
                    // if php can't create folder
                    else {
                        return(FALSE);
                    }
                } else {
                    // if ext folder is exist
                    return(TRUE);
                }
            }
        } else {
            // if param ext is null
            return(FALSE);
        }
    }

    /**
     * print site logo to images
     * @param   str     file name with full path
     * @param   str     file ext
     * @param   str     logo path
     * @return  boll    TRUE OR FALSE
     */
    public function printlogo($name, $ext, $logo) {

        if (!is_null($name) AND !is_null($ext) AND !is_null($logo) AND function_exists('gd_info')) {

            if (preg_match("/jpg|jpeg/", $ext)) {
                $src_img = imagecreatefromjpeg($name);

                if (!$src_img) {
                    return(FALSE);
                }
            }
            if (preg_match("/png/", $ext)) {
                $src_img = imagecreatefrompng($name);

                if (!$src_img) {
                    return(FALSE);
                }
            }

            if (preg_match("/gif/", $ext)) {
                $src_img = imagecreatefromgif($name);

                if (!$src_img) {
                    return(FALSE);
                }
            }

            $src_logo = imagecreatefrompng($logo);

            $bwidth = imageSX($src_img);
            $bheight = imageSY($src_img);
            $lwidth = imageSX($src_logo);
            $lheight = imageSY($src_logo);


            if ($bwidth > 160 && $bheight > 130) {

                $src_x = $bwidth - ($lwidth +10);
                $src_y = $bheight - ($lheight +10);
                ImageAlphaBlending($src_img, true);
                ImageCopy($src_img, $src_logo, $src_x, $src_y, 0, 0, $lwidth, $lheight);

                if (preg_match("/jpg|jpeg/", $ext)) {
                    imagejpeg($src_img, $name);
                }
                if (preg_match("/png/", $ext)) {
                    imagepng($src_img, $name);
                }
                if (preg_match("/gif/", $ext)) {
                    imagegif($src_img, $name);
                }
            } else {
                return(FALSE);
            }
        } else {
            return(FALSE);
        }
    }

    /**
     * create images thumbs
     * @param   str     file name with full path
     * @param   str     file ext
     * @param   str     thumbs file name
     * @param   int     thumbs width
     * @param   int     thumbs haight
     * @return  boll    TRUE OR FALSE
     */
    public function createthumb($name, $ext, $filename, $new_w, $new_h) {
        try {
            if (!is_null($name) && !is_null($ext) && !is_null($filename) && !is_null($new_w)
                    && !is_null($new_h) && function_exists('gd_info')) {

                // Create thumbs directory if it doesn't exist
                $thumbsDir = dirname($filename);
                if (!is_dir($thumbsDir)) {
                    if (!mkdir($thumbsDir, 0755, true)) {
                        error_log("Failed to create thumbs directory: " . $thumbsDir);
                        return false;
                    }
                }

                // Load source image based on extension
                $src_img = null;
                $ext = strtolower($ext);

                switch($ext) {
                    case 'jpg':
                    case 'jpeg':
                        $src_img = @imagecreatefromjpeg($name);
                        break;
                    case 'png':
                        $src_img = @imagecreatefrompng($name);
                        break;
                    case 'gif':
                        $src_img = @imagecreatefromgif($name);
                        break;
                    case 'webp':
                        $src_img = @imagecreatefromwebp($name);
                        break;
                }

                if (!$src_img) {
                    error_log("Failed to create source image from file: " . $name);
                    return false;
                }

                // Get original dimensions
                $old_x = imagesx($src_img);
                $old_y = imagesy($src_img);

                // Calculate aspect ratio if needed
                if ($new_w == 0) {
                    $new_w = floor($old_x * ($new_h / $old_y));
                }
                if ($new_h == 0) {
                    $new_h = floor($old_y * ($new_w / $old_x));
                }

                // Create destination image
                $dst_img = imagecreatetruecolor($new_w, $new_h);

                // Handle transparency for PNG and GIF
                if ($ext == 'png' || $ext == 'gif') {
                    imagealphablending($dst_img, false);
                    imagesavealpha($dst_img, true);
                    $transparent = imagecolorallocatealpha($dst_img, 255, 255, 255, 127);
                    imagefilledrectangle($dst_img, 0, 0, $new_w, $new_h, $transparent);
                }

                // Resize image
                if (!imagecopyresampled($dst_img, $src_img, 0, 0, 0, 0, $new_w, $new_h, $old_x, $old_y)) {
                    error_log("Failed to resize image");
                    return false;
                }

                // Save thumbnail based on extension
                $success = false;
                switch($ext) {
                    case 'jpg':
                    case 'jpeg':
                        $success = imagejpeg($dst_img, $filename, 85);
                        break;
                    case 'png':
                        $success = imagepng($dst_img, $filename, 8);
                        break;
                    case 'gif':
                        $success = imagegif($dst_img, $filename);
                        break;
                    case 'webp':
                        $success = imagewebp($dst_img, $filename, 85);
                        break;
                }

                // Clean up
                imagedestroy($dst_img);
                imagedestroy($src_img);

                if (!$success) {
                    error_log("Failed to save thumbnail: " . $filename);
                    return false;
                }

                return true;
            }
            return false;
        } catch (Exception $e) {
            error_log("Error creating thumbnail: " . $e->getMessage());
            return false;
        }
    }

    /**
     * get the mime type by ext
     * @param   str     file name with full path
     * @return  str     MIME type
     */
    public function Get_File_MimeType($filename, $tmp = '') {


        if (!is_null($filename)) {

            if (function_exists('finfo_open')& 1!=1) {
                $finfo = finfo_open(FILEINFO_MIME);
                $mimetype = finfo_file($finfo, $filename);
                finfo_close($finfo);
                return $mimetype;
            } else {

                $mime_types = array(
                    'txt' => 'text/plain',
                    'htm' => 'text/html',
                    'html' => 'text/html',
                    'php' => 'text/html',
                    'css' => 'text/css',
                    'js' => 'application/javascript',
                    'json' => 'application/json',
                    'xml' => 'application/xml',
                    'swf' => 'application/x-shockwave-flash',
                    'flv' => 'video/x-flv',
                    // images
                    'webp' => 'image/webp',
                    'png' => 'image/png',
                    'jpe' => 'image/jpeg',
                    'jpeg' => 'image/jpeg',
                    'jpg' => 'image/jpeg',
                    'gif' => 'image/gif',
                    'bmp' => 'image/bmp',
                    'ico' => 'image/vnd.microsoft.icon',
                    'tiff' => 'image/tiff',
                    'tif' => 'image/tiff',
                    'svg' => 'image/svg+xml',
                    'svgz' => 'image/svg+xml',
                    // archives
                    'zip' => 'application/zip',
                    'rar' => 'application/x-rar-compressed',
                    'exe' => 'application/x-msdownload',
                    'msi' => 'application/x-msdownload',
                    'cab' => 'application/vnd.ms-cab-compressed',
                    // audio/video
                    'mp3' => 'audio/mpeg',
                    'qt' => 'video/quicktime',
                    'mov' => 'video/quicktime',
                    'wmv' => 'video/x-ms-wmv',
                    'avi' => 'video/x-msvideo',
                    'wav' => 'audio/x-wav',
                    'ram' => 'audio/x-pn-realaudio',
                    '3gp' => 'video/3gpp',
                    'ra' => 'audio/vnd.rn-realaudio',
                    'ram' => 'audio/vnd.rn-realaudio',
                    'rm' => 'application/vnd.rn-realmedia',
                    'rpm' => 'audio/x-pn-realaudio-plugin',
                    // adobe
                    'pdf' => 'application/pdf',
                    'psd' => 'image/vnd.adobe.photoshop',
                    'ai' => 'application/postscript',
                    'eps' => 'application/postscript',
                    'ps' => 'application/postscript',
                    // ms office
                    'doc' => 'application/msword',
                    'rtf' => 'application/rtf',
                    'xls' => 'application/vnd.ms-excel',
                    'ppt' => 'application/vnd.ms-powerpoint',
                    // open office
                    'odt' => 'application/vnd.oasis.opendocument.text',
                    'ods' => 'application/vnd.oasis.opendocument.spreadsheet',
                    'docx' => 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
                );

                $ext = $this->GetExt($filename);
                if (array_key_exists($ext, $mime_types)) {

                    return $mime_types[$ext];
                } elseif (function_exists('mime_content_type')) {
                    return(mime_content_type($tmp));
                } else {
                    return 'application/octet-stream';
                }
            }
        } else {
            return(FALSE);
        }
    }

    /**
     * fix Ie MIME
     * @param   str     file mime type
     * @return  str
     */
    private function fixIeMIME($mime) {
        switch ($mime) {
            case"application/x-zip-compressed":
                return("application/zip");
                break;
            case"image/x-png":
                return("image/xpng");
                break;
            case"image/pjpeg":
                return("image/jpeg");
                break;
            case"audio/x-mpeg":
            case"audio/mp3":
            case"audio/x-mp3":
            case"audio/mpeg3":
            case"audio/x-mpeg3":
            case"audio/mpg":
            case"audio/x-mpg":
            case"audio/x-mpegaudio":
                return("audio/mpeg");
                break;
            default:
                return($mime);
                break;
        }
    }

    /**
     * chech file protection
     * @param   str     file mime
     * @param   str     file name
     * @return  boll    TRUE mean is positive file
     */
    public function protection($file_mimetype, $file_name, $tmp = '') {
        //fie IE mime type
        $file_mimetype = $this->fixIeMIME($file_mimetype);

        if (!is_null($file_mimetype) AND !is_null($file_name)) {
            $realmime = $this->Get_File_MimeType($file_name, $tmp);


            if (($realmime != $file_mimetype) AND ($file_mimetype != 'application/octet-stream')) {

                return(FALSE);
            } else {
                if (is_array($this->_disallow_mime)) {

                    if (in_array($file_mimetype, $this->_disallow_mime)) {
                        return(FALSE);
                    } else {
                        return(TRUE);
                    }
                } else {
                    return(TRUE);
                }
            }
        } else {

            return(FALSE);
        }
    }

    /**
     * set new error
     * @param   str     error
     */
    private function return_error($message) {
        return $this->error[] = $message;
    }

    public function showErrors() {
        if ($this->hasErrors()) { 
            reset($this->error);

            return($this->error);

            $this->resetErrors();
        }
    }

    public function hasErrors() {
        if (count($this->error) > 0) {
            return true;
        } else {
            return false;
        }
    }

    private function resetErrors() {
        if ($this->hasErrors()) {
            unset($this->error);
            $this->error = array();
        }
    }

    public function __destruct() {
        $this->_allowext = NULL;
        $this->_logoimages = FALSE;
        $this->_th_width = NULL;
        $this->_th_hight = NULL;
        $this->_nameprefix = NULL;
        $this->_maxsize = NULL;
        $this->_error = NULL;
        $this->_fileinfo = NULL;
        $this->_disallow_mime = NULL;
        $this->_logopath = NULL;
        $this->_uploadfolder = NULL;
    }

}

?>