<?php

require_once 'classes/studentRelationChecker.php';
require_once 'classes/teacherRelationChecker.php';
require_once 'classes/middleWareBase.php';

// Context class for middleware
class TestsMiddleware extends MiddlewareBase
{
    public function handleRequest(array $request): array
    {
        $relationChecker = $this->getRelationChecker();

        if (!$relationChecker->isRelatedToTest()) {
            return $this->returnErrorResponse($relationChecker->getErrorResponse());
        }

        return ['next' => true];
    }

    protected function getRelationChecker(): UserRelationCheckerInterface
    {
        switch ($this->userData['user_type']) {
            case 'student':
                return new StudentRelationChecker($this);
            case 'teacher':
                return new TeacherRelationChecker($this);
            default:
                return $this->handleTypelessUser();
        }
    }

    protected function handleTypelessUser(): UserRelationCheckerInterface
    {
        return new class($this) implements UserRelationCheckerInterface {
            private $middleware;

            public function __construct(MiddlewareBase $middleware)
            {
                $this->middleware = $middleware;
            }

            public function isRelatedToTest(): bool
            {
                $userIdHash = base64_decode($this->middleware->userData['user_id_hash']);
                return ($this->middleware->userData['user_id'] . $this->middleware->config['apihash'] === $userIdHash);
            }

            public function getErrorResponse(): array
            {
                return [
                    'message' => $this->middleware->lang['L_test'] . " " . $this->middleware->lang['L_removed'],
                    'on' => 'test_id',
                ];
            }
        };
    }
}

// Usage Example

