<?php

require_once('init.php');
require_once('/home/onlineapp/public_html/develop.testat-app.com/vendor/autoload.php');

use FFMpeg\Format\Video\X264;
use FFMpeg\FFMpeg;

$uploadFolderLink = $Config['uploads_path'] . $_GET['uploadFolderName'];

if ($_GET['uploadFolderName'] == 'course_videos_and_attachments') {
    $course_id = $_GET['course_id'];
    $uploadFolderLink = $uploadFolderLink . ($course_id ? '/' . $course_id : '');
    $uploadCourseFolderLink = $_SERVER['DOCUMENT_ROOT'] . "/" . $uploadFolderLink;

    if (!empty($course_id) && !is_dir($uploadCourseFolderLink)) {
        mkdir($uploadCourseFolderLink, 0777, true);
    }
}

$allowed_ext = [
    "mp3", "webp", "mp4", "mov", "wmv", "flv", "jpg", "gif", "png", "jpeg", "bmp", "pdf", "doc", "docm", "docx", "dot", "dotx"
];

$maxsize = 1024 * 1024 * 5; // 5GB
$upload = new Upload($allowed_ext, $th_width ?? null, $th_height ?? null, $maxsize, $uploadFolderLink, $thum ?? false);

header("Content-Type: application/json;charset=utf-8");

if (!empty($_FILES)) {
    foreach ($_FILES as $key => $fileDetails) {
        if (isset($fileDetails['name']) && $fileDetails['name'] != '') {
            $file = [
                'name' => addslashes($fileDetails["name"]),
                'type' => $fileDetails['type'],
                'size' => $fileDetails['size'] / 1024, // KB
                'tmp' => $fileDetails['tmp_name'],
                'ext' => $upload->GetExt($fileDetails["name"])
            ];

            $upfile = $upload->Upload_File($file, $maxsize);

            if ($upfile) {
                $response = [
                    'success' => true,
                    'file_name' => $upfile['newname']
                ];

                if ($_GET['uploadFolderName'] == 'course_videos_and_attachments') {
                    $allowed_video_extensions = ['mp4', 'avi', 'mov', 'mkv'];
                    if (in_array($file['ext'], $allowed_video_extensions)) {
                        $videoFilePath = $_SERVER['DOCUMENT_ROOT'] . '/' . $uploadFolderLink . '/' . $upfile['newname'];
                        $hlsFolder = $uploadCourseFolderLink . '/hls';

                        $hlsFilename = convertToHLSWithLibrary($videoFilePath, $hlsFolder);

                        if ($hlsFilename) {
                            $response['hls_file_name'] = $hlsFilename;
                            $response['time'] = getVideoDuration($videoFilePath);
                        } else {
                            $response['hls_conversion_error'] = 'Error converting to HLS.';
                        }
                    }
                }

                // Output each result directly
                echo json_encode($response) . PHP_EOL;
            } else {
                $errorResponse = [
                    'success' => false,
                    'message' => 'File upload failed.',
                    'errors' => $upload->showErrors()[0][0] ?? 'Unknown error'
                ];

                // Output error result directly
                echo json_encode($errorResponse) . PHP_EOL;
            }
        }
    }
} else {
    $errorResponse = [
        'success' => false,
        'message' => 'No file uploaded or file upload error.',
        'error_code' => $_FILES['file']['error'] ?? 'No error code'
    ];

    // Output error result directly
    echo json_encode($errorResponse) . PHP_EOL;
    die();
}

function convertToHLSWithLibrary($inputFile, $hlsFolder) {
    try {
        $ffmpeg = FFMpeg::create();
        
        if (!empty($hlsFolder) && !is_dir($hlsFolder)) {
            mkdir($hlsFolder, 0777, true);
        }

        $basename = basename($inputFile, '.' . pathinfo($inputFile, PATHINFO_EXTENSION));
        $hlsPath = $hlsFolder . "/" . $basename . '.m3u8';
        $segmentPattern = $hlsFolder . "/" . $basename . "_%03d.ts";

        $video = $ffmpeg->open($inputFile);
        $format = new X264('aac', 'libx264');
        $format->setAdditionalParameters([
            '-hls_time', '300',
            '-hls_list_size', '0',
            '-hls_segment_filename', $segmentPattern
        ]);

        $video->save($format, $hlsPath);

        return $basename . '.m3u8';
    } catch (Exception $e) {
        error_log('FFmpeg HLS conversion error: ' . $e->getMessage());
        return false;
    }
}

function getVideoDuration($videoPath) {
    $ffmpeg = FFMpeg::create();
    $video = $ffmpeg->open($videoPath);

    $durationInSeconds = $video->getFormat()->get('duration');

    $hours = floor($durationInSeconds / 3600);
    $minutes = floor(($durationInSeconds % 3600) / 60);
    $seconds = floor($durationInSeconds % 60);

    $formattedDuration = sprintf("%02d:%02d:%02d", $hours, $minutes, $seconds);

    return $formattedDuration; 
}
