<?php

require_once 'init.php';
require_once('/home/onlineapp/public_html/develop.testat-app.com/vendor/autoload.php');

use FFMpeg\Format\Video\X264;
use FFMpeg\FFMpeg;

if ($_GET['action'] == 'getAllVideosCourses') {
    if ($_SERVER['REQUEST_METHOD'] === 'GET') {

        $limit = isset($_GET['limit']) ? (int)$_GET['limit'] : 10; // Default to 10 if not provided
        $offset = isset($_GET['offset']) ? (int)$_GET['offset'] : 0; 

        $hlsFolderLink = $_SERVER['DOCUMENT_ROOT'] . "/api/uploads/course_videos_and_attachments/hls/";

        if (!is_dir($hlsFolderLink)) {
            mkdir($hlsFolderLink, 0777, true);
        }

        $allowed_video_extensions = ['mp4', 'avi', 'mov', 'mkv'];
        $Data = []; 

        $data = $videosCoursesObj->getAllVideosCourses($limit, $offset);
        //  $dataCount = $videosCoursesObj->getVideosCoursesCount();
        // print_r($data);print_r($dataCount);die();
        
        if (!$data) {
            $Data['error'] = "No more videos to process.";
        }

        foreach ($data as $file) {
            // Skip if the 'file' field is empty
            if (empty($file['file'])) {
                continue;
            }

            $extension = pathinfo($file['file'], PATHINFO_EXTENSION);

            if (in_array($extension, $allowed_video_extensions)) {
                $videoFilePath = $_SERVER['DOCUMENT_ROOT'] . "/api/uploads/course_videos_and_attachments/" . $file['file'];

                if (file_exists($videoFilePath)) {
                    $hlsFilename = convertToHLSWithLibrary($videoFilePath, $hlsFolderLink);
                    if ($hlsFilename) {
                        $updated = $videosCoursesObj->updateHLSFile($file['id'], $hlsFilename);

                        if ($updated) {
                            $Data['converted_files'][] = [
                                'id' => $file['id'],
                                'original_file' => $file['file'],
                                'hls_file' => $hlsFilename,
                            ];
                        } else {
                            $Data['update_errors'][] = [
                                'id' => $file['id'],
                                'file' => $file['file'],
                                'error' => 'Error updating HLS file in the database.',
                            ];
                        }
                    } else {
                        $Data['conversion_errors'][] = [
                            'id' => $file['id'],
                            'file' => $file['file'],
                            'error' => 'Error converting to HLS.',
                        ];
                    }
                } else {
                    $Data['missing_files'][] = [
                        'id' => $file['id'],
                        'file' => $file['file'],
                        'error' => 'File not found on the server.',
                    ];
                }
            }
        }

        header("Content-Type: application/json;charset=utf-8");
        echo json_encode($Data);
        die();
    }
}




function convertToHLSWithLibrary($videoFilePath, $hlsFolderLink) {
    try {
       
        if (!file_exists($videoFilePath)) {
            throw new Exception('Video file not found: ' . $videoFilePath);
        }

        if (!is_dir($hlsFolderLink)) {
            mkdir($hlsFolderLink, 0777, true);
        }

        $ffmpeg = FFMpeg::create();
        $video = $ffmpeg->open($videoFilePath);

        $basename = basename($videoFilePath, '.' . pathinfo($videoFilePath, PATHINFO_EXTENSION));
        $hlsFilename = $basename . '.m3u8';
        $hlsPath = $hlsFolderLink . $hlsFilename;

        $format = new \FFMpeg\Format\Video\X264();
        $format->setKiloBitrate(9000); 
        $format->setAudioCodec('aac'); 
    
        $video->save($format, $hlsPath);

        return $hlsFilename;

    } catch (Exception $e) {
        error_log('FFmpeg HLS conversion error: ' . $e->getMessage());
        return false;
    }
}

